package cn.com.qmth.stmms.api.utils;

import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.stmms.api.controller.PictureController;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.common.enums.LockType;

public class SheetDownloadThread implements Runnable {

    public static final Logger log = LoggerFactory.getLogger(SheetDownloadThread.class);

    public static final String PREFIX = "tag-sheet";

    private Integer examId;

    private File baseDir;

    private PictureController controller;

    private ExamStudentService studentService;

    private LockService lockService;

    public SheetDownloadThread(Integer examId, String dir, PictureController controller,
            ExamStudentService studentService, LockService lockService) {
        this.examId = examId;
        this.baseDir = new File(dir, PREFIX);
        this.controller = controller;
        this.studentService = studentService;
        this.lockService = lockService;
    }

    @Override
    public void run() {
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(examId);
        query.setUpload(true);
        query.setPageSize(100);
        query.setPageNumber(0);

        while (true) {
            query.setPageNumber(query.getPageNumber() + 1);
            query = studentService.findByQuery(query);
            if (query.getCurrentCount() == 0) {
                break;
            }

            for (ExamStudent student : query.getResult()) {
                for (int i = 1; i <= student.getSheetCount(); i++) {
                    download(student, i);
                }
            }
        }

        lockService.unlock(LockType.EXAM, examId);
    }

    private void download(ExamStudent student, int index) {
        File file = new File(baseDir, examId + File.separator + student.getSubjectCode() + File.separator
                + student.getExamNumber() + "-" + index + ".jpg");
        try {
            file.getParentFile().mkdirs();
            BufferedImage image = SheetBuildUtil.buildSheetImage(controller.getSheetImage(student, index),
                    studentService.buildSheetTags(student).get(index));
            ImageIO.write(image, "jpg", file);
        } catch (Exception e) {
            log.error("write sheet file error: " + file.getAbsolutePath(), e);
        }
    }

}
